#include <inttypes.h>

#define ledPin 13

#define mot1pwmPin  6
#define mot1dira1Pin 7
#define mot1dira2Pin 8
#define mot2pwmPin  5
#define mot2dira1Pin 4
#define mot2dira2Pin 3

#define DIR_COAST    3
#define DIR_BREAK    2
#define DIR_FORWARD  1
#define DIR_REVERSE  0
// tone table: thanks to IVO

//T�ne definieren - tiefere T�ne brummen und scheppern
#define G4              392 
#define A4		440
#define B4		466
#define H4		493
#define C5		523
#define DES5	554
#define D5		587
#define ES5		622
#define E5		659
#define F5		698
#define GES5	740
#define G5		783
#define AS5		830
#define A5		880
#define B5		932
#define H5		987
#define C6		1046
#define DES6	1108
#define D6		1174
#define ES6 	1244
#define E6		1318
#define F6		1396
#define GES6	1479
#define G6		1567
#define AS6		1661
#define A6		1760
#define B6		1864
#define H6		1975
#define C7		2093
#define DES7	2217
#define D7		2349
#define ES7		2489
#define E7		2637
#define F7		2793
#define GES7	2959
#define G7		3135
#define AS7		3322
#define A7		3520
#define H7		3729
#define B7		3951

#define QUAD 250
#define HALF 500
#define FULL 1000
#define _OFF 0
#define STOP 0

uint16_t frere_jaques[] = {
	D5,HALF,
	E5,HALF,
	F5,HALF,
	D5,HALF,
	D5,HALF,
	E5,HALF,
	F5,HALF,
	D5,HALF,

	F5,HALF,
	G5,HALF,
	A5,FULL,
	F5,HALF,
	G5,HALF,
	A5,FULL,

	A5,QUAD,
	H5,QUAD,
	A5,QUAD,
	G5,QUAD,
	F5,HALF,
	D5,HALF,
	A5,QUAD,
	H5,QUAD,
	A5,QUAD,
	G5,QUAD,
	F5,HALF,
	D5,HALF,

	D5,HALF,
	A4,HALF,
	D5,FULL,
	D5,HALF,
	A4,HALF,
	D5,FULL,
	STOP,STOP
};  

void playMelody(uint16_t *notes)
{
  int index = 0;
  uint16_t note;
  uint16_t dur;
  
  do
  {
    note = notes[index++];
    dur = notes[index++];
    playSound(note, dur, 255);
  } while (dur != 0);

}

void playSound(uint16_t freq, uint16_t duration_msec, uint8_t  amplitude)
{
  uint32_t n,k,period_usec,dauer_usec;

  period_usec = 1000000L / freq;
  dauer_usec = 1000 * (uint32_t) duration_msec;
  k = dauer_usec / period_usec;

  motorSpeed (amplitude, amplitude);

  for (n = 0; n < k; n++)
  {
    motorDirection (DIR_FORWARD, DIR_FORWARD);
    delayMicroseconds(period_usec/2);
    motorDirection (DIR_REVERSE, DIR_REVERSE);
    delayMicroseconds(period_usec/2);
  }
  motorSpeed (0, 0);
}

#define BEEP playSound(1000, 100, 255)

void motorInit(void)
{
  pinMode(mot1dira1Pin, OUTPUT);
  pinMode(mot2dira1Pin, OUTPUT);
  pinMode(mot1dira2Pin, OUTPUT);
  pinMode(mot2dira2Pin, OUTPUT);
  analogWrite(mot1pwmPin, 0);
  digitalWrite(mot1dira1Pin, 0);
  digitalWrite(mot1dira2Pin, 0);
  analogWrite(mot2pwmPin, 0);
  digitalWrite(mot2dira1Pin, 0);
  digitalWrite(mot2dira2Pin, 0);
}

void motorSpeed(int speedLt, int speedRt)
{
  analogWrite(mot1pwmPin, speedLt);
  analogWrite(mot2pwmPin, speedRt);   
}

void motorDirection(int dirLt, int dirRt)
{
  switch(dirLt)
  {
    case DIR_FORWARD:
      digitalWrite(mot1dira1Pin, HIGH);
      digitalWrite(mot1dira2Pin, LOW);
    break;
    case DIR_REVERSE:
      digitalWrite(mot1dira1Pin, LOW);
      digitalWrite(mot1dira2Pin, HIGH);
    break;  
    case DIR_BREAK:
      digitalWrite(mot1dira1Pin, HIGH);
      digitalWrite(mot1dira2Pin, HIGH);
    break;  
    case DIR_COAST:
      digitalWrite(mot1dira1Pin, LOW);
      digitalWrite(mot1dira2Pin, LOW);
    break;  
  }
  switch(dirRt)
  {
    case DIR_FORWARD:
      digitalWrite(mot2dira1Pin, HIGH);
      digitalWrite(mot2dira2Pin, LOW);
    break;
    case DIR_REVERSE:
      digitalWrite(mot2dira1Pin, LOW);
      digitalWrite(mot2dira2Pin, HIGH);
    break;  
    case DIR_BREAK:
      digitalWrite(mot2dira1Pin, HIGH);
      digitalWrite(mot2dira2Pin, HIGH);
    break;  
    case DIR_COAST:
      digitalWrite(mot2dira1Pin, LOW);
      digitalWrite(mot2dira2Pin, LOW);
    break;  
  }
}

void blinkLed(int cnt, int duration)
{
  int i;
  for(i=0; i<cnt; i++)
  {
    digitalWrite(ledPin, HIGH);
    delay(duration/2);
    digitalWrite(ledPin, LOW);
    delay(duration/2);
  }
}

SIGNAL (SIG_OVERFLOW2)
{
  TCNT2 += 0x25;
}

void setup() 
{ 
  pinMode(ledPin, OUTPUT);
  motorInit();
  Serial.begin(38400);
  Serial.println("L293D 3-pin");
  blinkLed(3, 1000);
  BEEP;
  delay(1000);
  BEEP;
  delay(1000);
  BEEP;
  delay(1000);
}


void loop()
{
  // One voice on two notors
  playMelody(frere_jaques);
  delay(2000);
}

